try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from AccessControl import ClassSecurityInfo
from Products.PloneHelpCenter.config import *
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import safe_hasattr
from AccessControl import getSecurityManager

class PHCFolder:
    """A simple mixin for folderish archetype"""

    _at_rename_after_creation = True

    actions = (
        {'id'          : 'view',
         'name'        : 'View',
         'action'      : 'string:${object_url}',
         'permissions' : (CMFCorePermissions.View,)
         },
        {'id'          : 'edit',
         'name'        : 'Edit',
         'action'      : 'string:${object_url}/edit',
         'permissions' : (CMFCorePermissions.ModifyPortalContent,),
         },
        {'id'          : 'metadata',
         'name'        : 'Properties',
         'action'      : 'string:${object_url}/properties',
         'permissions' : (CMFCorePermissions.ModifyPortalContent,),
         },
        {'id'          : 'local_roles',
         'name'        : 'Sharing',
         'action'      : 'string:${object_url}/sharing',
         'permissions' : (CMFCorePermissions.ManageProperties,),
         },
        {'id'          : 'stats',
         'name'        : 'Statistics',
         'action'      : 'string:${object_url}/stats',
         'permissions' : (CMFCorePermissions.ManageProperties,),
         },
         
        )

    aliases = {
        # Set (Default) and view in each type
        # '(Default)'  : '',
        # 'view'       : '',
        'index.html' : '',
        'edit'       : 'base_edit',
        'properties' : 'base_metadata',
        'sharing'    : 'folder_localrole_form',
        'stats'      : 'phc_stats',
        'gethtml'    : '',
        'mkdir'      : '',
        }

    security = ClassSecurityInfo()

    security.declareProtected(CMFCorePermissions.View, 'getVersionsVocab')
    def getVersionsVocab(self):
        """Get version vocabulary."""
        if not safe_hasattr(self.aq_parent, 'getVersionsVocab'):
            return []
        return self.aq_parent.getVersionsVocab()
            
    security.declareProtected(CMFCorePermissions.View, 'getAudiencesVocab')
    def getAudiencesVocab(self):
        """Get sections vocabulary."""
        if not safe_hasattr(self.aq_parent, 'getAudiencesVocab'):
            return []
        return self.aq_parent.getAudiencesVocab()

    security.declareProtected(CMFCorePermissions.View, 'Versions')
    def Versions(self):
        """Method to display the versions in a nicer way."""
        return ", ".join(getattr(self, 'versions', []))

    security.declareProtected(CMFCorePermissions.View, 'getItems')
    def getItems(self, states=[], audiences=[]):
        """Get items in this folder, possibly filtered on states, 
        sections and/or audience
        """
        criteria = {}
        if states:
            criteria['review_state'] = states
        if audiences:
            criteria['getAudiences'] = audiences
        return self.getFolderContents(contentFilter = criteria)


    security.declareProtected(CMFCorePermissions.View, 'getItemsBySection')
    def getItemsBySection(self, section, states=[], audiences=[]):
        """Get items in this section"""
        criteria = {'getSections' : section}
        if states:
            criteria['review_state'] = states
        if audiences:
            criteria['getAudiences'] = audiences
        return self.getFolderContents(contentFilter = criteria)


    security.declareProtected(CMFCorePermissions.View, 'getItemsByAudiencesAndSections')
    def getItemsByAudiencesAndSections(self, states=[]):
        """Get all items to list, by audience and section. Returns a list of
        dicts:
            
            'audience'  : The name of the audience category
            'sections'  : A list of dicts:
            
                'id'      : A normalised string representing the section
                'section' : The name of the section
                'items'   : A list of catalog brains for items in this section
                
        The first item will have an 'audience' title of 'Any audience' and 
        contain all items with no audience selected.
        """
        plone_utils = getToolByName(self, 'plone_utils')
        brains = self.getFolderContents(contentFilter = {'review_state' : states})
        
        # Set up the dicts listing all audiences + sections
        # This is needed because we want it to list the audiences and sections
        # in the order the vocab specifies them.
        audiences = []
        for a in ['Any audience'] + list(self.getAudiencesVocab()):
            sections = []
            for s in self.getSectionsVocab():
                sections.append({'id'      : plone_utils.normalizeString(s),
                                 'section' : s,
                                 'items'   : []})
            audiences.append({'audience' : a,
                              'sections' : sections})
                              
        # Then insert each how-to in the appropriate audience/section
        for b in brains:
            itemAudiences = b.getAudiences or ['Any audience']            
            matchedAudiences = [a for a in audiences if a['audience'] in itemAudiences]
            for a in matchedAudiences:
                matchedSections = [s for s in a['sections'] if s['section'] in b.getSections]
                for s in matchedSections:
                    s['items'].append(b)

        # Finally clean out empty audiences or sections
        delAudiences = []
        for i in range(len(audiences)):
            a = audiences[i]
            delSections = []
            for j in range(len(a['sections'])):
                if len(a['sections'][j]['items']) == 0:
                    delSections.append(j)
            delSections.reverse()
            for j in delSections:
                del a['sections'][j]
            if len(a['sections']) == 0:
                delAudiences.append(i)
        delAudiences.reverse()
        for i in delAudiences:
            del audiences[i]
        return audiences

    security.declareProtected(CMFCorePermissions.View, 'getSectionsToList')
    def getSectionsToList(self, states=[]):
        """Sections that have at least one listable item. Note that this does 
        not take account of audiences.
        """

        sections = {}
        allSections = self.getSectionsVocab()
        max_sections = len(allSections)

        for o in self.getItems(states):
            for s in o.getSections:
                sections[s]=1
            if len(sections) == max_sections:
                break
        return [s for s in allSections if sections.has_key(s)]